% example code of ensemble projection
% written by Dengxin Dai dai@vision.ee.ethz.ch
% Computer Vision Lab, ETH Zurich
% Sept. 2013

clear all, clc; 
addpath(genpath(pwd)); 

set_name = '25';  % dataset_name, {'15', '21', '25',  '101'}
fet_name = 'phog'; % feature_name {'gist', 'phog', 'lbp', 'all'}, where all is the concatenation of gist, phog, and lbp
cla_name = 'lr'; % classifier_name {'knn', 'lr', 'svm'}, where lr is logistic regression

data_dir = './data/';    
fet_dir = './feature/';

%% loading data

fet_mms = []; %feature for max-min sampling    (defining neighborhoods)
fet_ep  = []; %feature for ensemble projection (learning projection functions)

if ~strcmp(fet_name,'all')                   % a single feature      
    
    data_name = strcat( data_dir, set_name, '_', fet_name, '.mat');        
    load(data_name);
    fet_mms =  data.fet;                    % the  feature itself for mms
    fet_ep  =  data.fet;                    % the feature itself for ep    
   
else                                        % all features
  
    data_name = strcat( data_dir, set_name, '_', 'gist', '.mat'); 
    load(data_name);
    
    fet_mms =  data.fet;                    % gist for mms
    fet_ep  =  cat(2, fet_ep, data.fet);    % the concatenation of the 3 fets for ep
    
    data_name = strcat( data_dir, set_name, '_', 'phog', '.mat'); 
    load(data_name);
    fet_ep  =  cat(2, fet_ep, data.fet);                     

    data_name = strcat( data_dir, set_name, '_', 'lbp', '.mat'); 
    load(data_name);
    fet_ep  =  cat(2, fet_ep, data.fet);             
end


class_num = length(unique(data.lab));      %the number of classes

%% parameters;  
r = 30;                                    %the number of prototypes 
n = 6;                                     %the number of images in each prototypes
T = 300;                                   %the number of prototype sets
m = 50;                                    %the number of hypotheses in Max-min sampling 

%% feature by ensemble projection 
% if ~exist(strcat(fet_dir, 'EP_', set_name, '_', fet_name, '.mat'))
    
    [Models] = ProjectFunction(fet_ep, r, n, T, m, fet_mms);           % all projection functions
    
    EP = EPFeatureComputing(Models, fet_ep);                           % projected values as features
%     save(strcat(fet_dir, 'EP_', set_name, '_', fet_name, '.mat'), 'EP'); 
% else 
%    load(strcat(fet_dir, 'EP_', set_name, '_', fet_name, '.mat')); 
% end
%% training & test data
train_num = 5;                          % the number of labeled training data per class 
train_index = [];                       % index of labeled training data
test_index = [];                        % index of unlabeled training (test) data

for c=1:class_num
    index_c = find(data.lab==c); 
    randIndex = randperm(length(index_c));
    train_index = cat(1, train_index, index_c(randIndex(1:train_num))); 
    test_index = cat(1, test_index, index_c(randIndex(train_num+1:end)));
end
%% classification with three classifiers: knn, logistic regression, and linear svms 
switch cla_name

    case 'knn'
          ap_ori = KNNClassification(fet_ep, train_index, test_index, data.lab, class_num, train_num); 
          ap_ep  = KNNClassification(EP, train_index, test_index, data.lab, class_num, train_num); 
          
    case 'lr'
          ap_ori = LRClassification(fet_ep, train_index, test_index, data.lab, class_num); 
          ap_ep  = LRClassification(EP, train_index, test_index, data.lab, class_num); 
    case 'svm'
          ap_ori = SVMClassification(fet_ep, train_index, test_index, data.lab, class_num); 
          ap_ep  = SVMClassification(EP, train_index, test_index, data.lab, class_num); 
end

[ap_ori, ap_ep]
